@@ -1,6 +1,7 @@
 package cmd
 
 import (
+	"bytes"
 	"errors"
 	"fmt"
 	"github.com/sibprogrammer/xq/internal/utils"
@@ -43,17 +44,24 @@ var rootCmd = &cobra.Command{
 			}
 		}
 
-		query, singleNode := getXpathQuery(cmd.Flags())
+		xPathQuery, singleNode := getXpathQuery(cmd.Flags())
+		cssQuery, _ := cmd.Flags().GetString("query")
+
 		pr, pw := io.Pipe()
 
 		go func() {
 			defer pw.Close()
 
-			if query != "" {
-				err = utils.XPathQuery(reader, pw, query, singleNode)
+			if xPathQuery != "" {
+				err = utils.XPathQuery(reader, pw, xPathQuery, singleNode)
+			} else if cssQuery != "" {
+				err = utils.CSSQuery(reader, pw, cssQuery)
 			} else {
 				colors := getColorMode(cmd.Flags())
-				isHtmlFormatter, _ := cmd.Flags().GetBool("html")
+
+				var isHtmlFormatter bool
+				isHtmlFormatter, reader = isHtmlFormatterNeeded(cmd.Flags(), reader)
+
 				if isHtmlFormatter {
 					err = utils.FormatHtml(reader, pw, indent, colors)
 				} else {
@@ -84,10 +92,14 @@ func Execute() {
 	rootCmd.PersistentFlags().StringP("xpath", "x", "", "Extract the node(s) from XML")
 	rootCmd.PersistentFlags().StringP("extract", "e", "", "Extract a single node from XML")
 	rootCmd.PersistentFlags().Bool("tab", viper.GetBool("tab"), "Use tabs for indentation")
-	rootCmd.PersistentFlags().Int("indent", viper.GetInt("indent"), "Use the given number of spaces for indentation")
+	rootCmd.PersistentFlags().Int("indent", viper.GetInt("indent"),
+		"Use the given number of spaces for indentation")
 	rootCmd.PersistentFlags().Bool("no-color", viper.GetBool("no-color"), "Disable colorful output")
-	rootCmd.PersistentFlags().BoolP("color", "c", viper.GetBool("color"), "Force colorful output")
+	rootCmd.PersistentFlags().BoolP("color", "c", viper.GetBool("color"),
+		"Force colorful output")
 	rootCmd.PersistentFlags().BoolP("html", "m", viper.GetBool("html"), "Use HTML formatter")
+	rootCmd.PersistentFlags().StringP("query", "q", "",
+		"Extract the node(s) using CSS selectors")
 
 	if err := rootCmd.Execute(); err != nil {
 		os.Exit(1)
@@ -163,3 +175,24 @@ func getColorMode(flags *pflag.FlagSet) int {
 
 	return colors
 }
+
+func isHtmlFormatterNeeded(flags *pflag.FlagSet, origReader io.Reader) (bool, io.Reader) {
+	isHtmlFormatter, _ := flags.GetBool("html")
+	if isHtmlFormatter {
+		return isHtmlFormatter, origReader
+	}
+
+	buf := make([]byte, 10)
+	length, err := origReader.Read(buf)
+	if err != nil {
+		return false, origReader
+	}
+
+	prefix := strings.ToLower(string(buf))
+	if strings.Contains(prefix, "html") || strings.Contains(prefix, "<!d") {
+		isHtmlFormatter = true
+	}
+
+	reader := io.MultiReader(bytes.NewReader(buf[:length]), origReader)
+	return isHtmlFormatter, reader
+}
