@@ -3,6 +3,7 @@ package utils
 import (
 	"encoding/xml"
 	"fmt"
+	"github.com/PuerkitoBio/goquery"
 	"github.com/antchfx/xmlquery"
 	"github.com/fatih/color"
 	"golang.org/x/net/html"
@@ -22,6 +23,8 @@ const (
 
 func FormatXml(reader io.Reader, writer io.Writer, indent string, colors int) error {
 	decoder := xml.NewDecoder(reader)
+	decoder.Strict = false
+	decoder.AutoClose = xml.HTMLAutoClose
 	decoder.CharsetReader = func(charset string, input io.Reader) (io.Reader, error) {
 		e, err := ianaindex.MIME.Encoding(charset)
 		if err != nil {
@@ -96,7 +99,9 @@ func FormatXml(reader io.Reader, writer io.Writer, indent string, colors int) er
 			level++
 		case xml.CharData:
 			str := string(typedToken)
-			str = strings.TrimSpace(str)
+			if strings.TrimSpace(str) == "" {
+				str = ""
+			}
 			hasContent = str != ""
 			if hasContent && !startTagClosed {
 				_, _ = fmt.Fprint(writer, tagColor(">"))
@@ -144,24 +149,42 @@ func FormatXml(reader io.Reader, writer io.Writer, indent string, colors int) er
 }
 
 func XPathQuery(reader io.Reader, writer io.Writer, query string, singleNode bool) error {
-	doc, err := xmlquery.Parse(reader)
+	doc, err := xmlquery.ParseWithOptions(reader, xmlquery.ParserOptions{
+		Decoder: &xmlquery.DecoderOptions{
+			Strict:    false,
+			AutoClose: xml.HTMLAutoClose,
+		},
+	})
 	if err != nil {
 		return err
 	}
 
 	if singleNode {
 		if n := xmlquery.FindOne(doc, query); n != nil {
-			_, _ = fmt.Fprintf(writer, "%s\n", n.InnerText())
+			_, _ = fmt.Fprintf(writer, "%s\n", strings.TrimSpace(n.InnerText()))
 		}
 	} else {
 		for _, n := range xmlquery.Find(doc, query) {
-			_, _ = fmt.Fprintf(writer, "%s\n", n.InnerText())
+			_, _ = fmt.Fprintf(writer, "%s\n", strings.TrimSpace(n.InnerText()))
 		}
 	}
 
 	return nil
 }
 
+func CSSQuery(reader io.Reader, writer io.Writer, query string) error {
+	doc, err := goquery.NewDocumentFromReader(reader)
+	if err != nil {
+		return err
+	}
+
+	doc.Find(query).Each(func(index int, item *goquery.Selection) {
+		_, _ = fmt.Fprintf(writer, "%s\n", strings.TrimSpace(item.Text()))
+	})
+
+	return nil
+}
+
 func FormatHtml(reader io.Reader, writer io.Writer, indent string, colors int) error {
 	tokenizer := html.NewTokenizer(reader)
 
@@ -175,6 +198,7 @@ func FormatHtml(reader io.Reader, writer io.Writer, indent string, colors int) e
 
 	level := 0
 	hasContent := false
+	forceNewLine := false
 	selfClosingTags := getSelfClosingTags()
 
 	for {
@@ -191,7 +215,9 @@ func FormatHtml(reader io.Reader, writer io.Writer, indent string, colors int) e
 		switch token {
 		case html.TextToken:
 			str := string(tokenizer.Text())
-			str = strings.TrimSpace(str)
+			if strings.TrimSpace(str) == "" {
+				str = ""
+			}
 			hasContent = str != ""
 			_, _ = fmt.Fprint(writer, str)
 		case html.StartTagToken, html.SelfClosingTagToken:
@@ -228,26 +254,28 @@ func FormatHtml(reader io.Reader, writer io.Writer, indent string, colors int) e
 			} else {
 				level++
 				_, _ = fmt.Fprint(writer, tagColor(">"))
+				forceNewLine = false
 			}
 		case html.EndTagToken:
 			level--
 			tagName, _ := tokenizer.TagName()
 
-			if !hasContent {
+			if forceNewLine {
 				_, _ = fmt.Fprint(writer, "\n", strings.Repeat(indent, level))
 			}
 			_, _ = fmt.Fprint(writer, tagColor("</"+string(tagName)+">"))
 
 			hasContent = false
+			forceNewLine = true
 		case html.DoctypeToken:
 			docType := tokenizer.Text()
 			_, _ = fmt.Fprintf(writer, "%s%s%s\n", tagColor("<!doctype "), string(docType), tagColor(">"))
 		case html.CommentToken:
-			comment := tokenizer.Text()
+			comment := tokenizer.Raw()
 			if !hasContent && level > 0 {
 				_, _ = fmt.Fprint(writer, "\n", strings.Repeat(indent, level))
 			}
-			_, _ = fmt.Fprint(writer, commentColor("<!--"+string(comment)+"-->"))
+			_, _ = fmt.Fprint(writer, commentColor(string(comment)))
 			if level == 0 {
 				_, _ = fmt.Fprint(writer, "\n")
 			}
